<?php
namespace App\Factory;

use Doctrine\DBAL\Connection;

class Address {
    private $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }
    
    public function addAdditionalAddress($params, $addressCode)
	{
		$sql = "
		INSERT INTO ENDERECOCONS(
			CDENDECONS, CDCLIENTE, CDCONSUMIDOR, NOMEENDE,
			NRCEPCONS, CDPAIS, SGESTADO, CDMUNICIPIO, CDBAIRRO,
			DSENDECONS, NRENDECONS, DSCOMPLENDECONS, DSREFERENDECONS,
			NRLATENDE, NRLONGENDE, NMBAIRCONS
		)
		VALUES (
			:CDENDECONS, :CDCLIENTE, :CDCONSUMIDOR, :NOMEENDE,
			:NRCEPCONS, '0055', :SGESTADO, :CDMUNICIPIO, :CDBAIRRO,
			:DSENDECONS, :NRENDECONS, :DSCOMPLENDECONS, :DSREFERENDECONS,
			:NRLATENDE, :NRLONGENDE, :NMBAIRCONS
		)";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue("CDENDECONS", $addressCode);
		$stmt->bindValue("CDCLIENTE", $params['CDCLIENTE']);
		$stmt->bindValue("CDCONSUMIDOR", $params['CDCONSUMIDOR']);
		$stmt->bindValue("NOMEENDE", $params['NOMEENDE']);
		$stmt->bindValue("NRCEPCONS", $params['NRCEPCONS']);
		$stmt->bindValue("SGESTADO", $params['SGESTADO']);
		$stmt->bindValue("CDMUNICIPIO", $params['CDMUNICIPIO']);
		$stmt->bindValue("CDBAIRRO", $params['CDBAIRRO']);
		$stmt->bindValue("DSENDECONS", $params['DSENDECONS']);
		$stmt->bindValue("NRENDECONS", $params['NRENDECONS']);
		$stmt->bindValue("DSCOMPLENDECONS", $params['DSCOMPLENDECONS']);
		$stmt->bindValue("DSREFERENDECONS", $params['DSREFERENDECONS']);
		$stmt->bindValue("NRLATENDE", $params['NRLATENDE']);
		$stmt->bindValue("NRLONGENDE", $params['NRLONGENDE']);
		$stmt->bindValue("NMBAIRCONS", $params['NMBAIRCONS']);

		$result = $stmt->executeQuery();
		
    	return $result;
	}

	public function deleteAdditionalAddress($params)
	{
		$sql = "
		DELETE FROM ENDERECOCONS
		      WHERE CDCLIENTE    = :CDCLIENTE
		        AND CDCONSUMIDOR = :CDCONSUMIDOR
		        AND CDENDECONS   = :CDENDECONS
		";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue("CDENDECONS", $params['CDENDECONS']);
		$stmt->bindValue("CDCLIENTE", $params['CDCLIENTE']);
		$stmt->bindValue("CDCONSUMIDOR", $params['CDCONSUMIDOR']);

		$result = $stmt->executeQuery();
		
    	return $result;
	}

	public function getNeighbourhoodCode($NMBAIRRO)
	{
		$sql = "
			SELECT CDBAIRRO
			FROM BAIRRO
			WHERE UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)
		";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue("NMBAIRRO", $NMBAIRRO);

		$result = $stmt->executeQuery();
		
    	return $result->fetchAssociative();
	}

	public function getCityCode($NMMUNICIPIO)
	{
		$sql = "
			SELECT CDMUNICIPIO
			FROM MUNICIPIO
			WHERE UPPER(NMMUNICIPIO) = UPPER(:NMMUNICIPIO)
		";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue("NMMUNICIPIO", $NMMUNICIPIO);

		$result = $stmt->executeQuery();
		
    	return $result->fetchAssociative();
	}

	public function updateAddress($params) {
		$sql = "
			UPDATE CONSUMIDOR
			SET DSEXIBENDE      = :NOMEENDE,
				NRCEPCONS       = :NRCEPCONS,
				DSENDECONS      = :DSENDECONS,
				DSCOMPLENDECONS = :DSCOMPLENDECONS,
				DSREFERENDECONS = :DSREFERENDECONS,
				NRENDECONS      = :NRENDECONS,
				NRLATCONS       = :NRLATCONS,
				NRLONGCONS      = :NRLONGCONS,
				NMBAIRCONS	   = :NMBAIRCONS
			WHERE CDCLIENTE       = :CDCLIENTE
				AND CDCONSUMIDOR    = :CDCONSUMIDOR
		";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue(':CDCLIENTE', $params['CDCLIENTE']);
		$stmt->bindValue(':CDCONSUMIDOR', $params['CDCONSUMIDOR']);
		$stmt->bindValue(':NOMEENDE', $params['NOMEENDE']);
		$stmt->bindValue(':NRCEPCONS', $params['NRCEPCONS']);
		$stmt->bindValue(':DSENDECONS', $params['DSENDECONS']);
		$stmt->bindValue(':DSCOMPLENDECONS', $params['DSCOMPLENDECONS']);
		$stmt->bindValue(':DSREFERENDECONS', $params['DSREFERENDECONS']);
		$stmt->bindValue(':NRENDECONS', $params['NRENDECONS']);
		$stmt->bindValue(':NRLATCONS', $params['NRLATENDE']);
		$stmt->bindValue(':NRLONGCONS', $params['NRLONGENDE']);
		$stmt->bindValue(':NMBAIRCONS', $params['NMBAIRCONS']);
		
		return $stmt->executeQuery(); 
	}

	public function updateAdditionalAddress($params) {
		$sql = "
			UPDATE ENDERECOCONS
				SET NOMEENDE        = :NOMEENDE,
					NRCEPCONS       = :NRCEPCONS,
					DSENDECONS      = :DSENDECONS,
					DSCOMPLENDECONS = :DSCOMPLENDECONS,
					DSREFERENDECONS = :DSREFERENDECONS,
					NRENDECONS      = :NRENDECONS,
					NRLATENDE       = :NRLATENDE,
					NRLONGENDE      = :NRLONGENDE,
					NMBAIRCONS	   = :NMBAIRCONS
			WHERE CDCLIENTE       = :CDCLIENTE
				AND CDCONSUMIDOR    = :CDCONSUMIDOR
				AND CDENDECONS      = :CDENDECONS
		";

		$stmt = $this->connection->prepare($sql);
		
		$stmt->bindValue(':CDCLIENTE', $params['CDCLIENTE']);
		$stmt->bindValue(':CDCONSUMIDOR', $params['CDCONSUMIDOR']);
		$stmt->bindValue(':CDENDECONS', $params['CDENDECONS']);
		$stmt->bindValue(':NOMEENDE', $params['NOMEENDE']);
		$stmt->bindValue(':NRCEPCONS', $params['NRCEPCONS']);
		$stmt->bindValue(':DSENDECONS', $params['DSENDECONS']);
		$stmt->bindValue(':DSCOMPLENDECONS', $params['DSCOMPLENDECONS']);
		$stmt->bindValue(':DSREFERENDECONS', $params['DSREFERENDECONS']);
		$stmt->bindValue(':NRENDECONS', $params['NRENDECONS']);
		$stmt->bindValue(':NRLATENDE', $params['NRLATENDE']);
		$stmt->bindValue(':NRLONGENDE', $params['NRLONGENDE']);
		$stmt->bindValue(':NMBAIRCONS', $params['NMBAIRCONS']);
		
		return $stmt->executeQuery(); 
	}
}