<?php

namespace App\Factory;

use Doctrine\DBAL\Connection;

class Integrations
{
    private $connection;

    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }

    public function getSubsidiaryIdToken($params) {
      $sql = "
        SELECT
            CDFILIAL,
            IDTOKEN,
            NVL(DSTOKENFILI, 'N/A') ||' - '|| IDTOKEN AS FILIAL
        FROM
            INTEGRACAOFILI
        WHERE
            CDINTEGRACAO = :CDINTEGRACAO AND CDFILIAL = :CDFILIAL
        ORDER BY
            FILIAL ASC
      ";

      $stmt = $this->connection->prepare($sql);
      $stmt->bindValue("CDFILIAL", $params["cdFilial"]);
      $stmt->bindValue("CDINTEGRACAO", $params["cdIntegracao"]);
      $result = $stmt->executeQuery();
      return $result->fetchAllAssociative();
    }

    public function getCdIntegracaoByIdIntegracao($IDINTEGRACAO) {
        $sql = "
            SELECT
                CDINTEGRACAO
            FROM
                INTEGRACAO
            WHERE
                IDINTEGRACAO = :IDINTEGRACAO
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDINTEGRACAO", $IDINTEGRACAO);
        $result = $stmt->executeQuery();
        $result = $result->fetchAssociative();
        return $result['CDINTEGRACAO'];
    }

    public function getProducts($search, $page, $limit) {
        $sqlTotalRegistros = "
        SELECT COUNT(*) as TOTAL
        FROM
            PRODUTO
        WHERE
            LOWER(NMPRODUTO) LIKE '%' || LOWER(:NMPRODUTO) || '%'
        AND
            LENGTH(CDPRODUTO) = 10
        ";

        $stmtTotalRegistros = $this->connection->prepare($sqlTotalRegistros);
        $stmtTotalRegistros->bindValue("NMPRODUTO", $search);;
        $totalRegistros = $stmtTotalRegistros->executeQuery();
        $totalRegistros = $totalRegistros->fetchAllAssociative();

        $sql = "
        SELECT
            NMPRODUTO,
            CDPRODUTO
        FROM
            PRODUTO
        WHERE
            LOWER(NMPRODUTO) LIKE '%' || LOWER(:NMPRODUTO) || '%'
        AND
            LENGTH(CDPRODUTO) = 10
        OFFSET (:PAGE-1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("NMPRODUTO", $search);
        $stmt->bindValue("PAGE", $page);
        $stmt->bindValue("LIMIT", $limit);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        return [$result, $totalRegistros];
    }

    public function getOccurrences($params) {
        $sqlTotal =  "
        SELECT COUNT(*) as TOTAL
        FROM
            OCORRENCIA
            WHERE
            LOWER(DSOCORR) LIKE '%' || LOWER(:DSOCORR) || '%'
        AND (
            (IDCONTROLAOBS = 'A' AND :IDCONTROLAOBS = 'A')
            OR
            IDCONTROLAOBS IN ('R', 'O') AND :IDCONTROLAOBS <> 'A'
        )
        ";
        $stmtTotal= $this->connection->prepare($sqlTotal);
        $stmtTotal->bindValue("DSOCORR", $params['search']);
        $stmtTotal->bindValue("IDCONTROLAOBS", $params['idControlaObs']);
        $total = $stmtTotal->executeQuery();
        $total= $total->fetchAllAssociative();

        $sql = "
        SELECT
            DSOCORR AS NMPRODUTO,
            CDGRPOCOR || '|' || CDOCORR AS CDPRODUTO,
            IDCONTROLAOBS
        FROM
            OCORRENCIA
        WHERE
            LOWER(DSOCORR) LIKE '%' || LOWER(:DSOCORR) || '%'
        AND (
            (IDCONTROLAOBS = 'A' AND :IDCONTROLAOBS = 'A')
            OR
            IDCONTROLAOBS IN ('R', 'O') AND :IDCONTROLAOBS <> 'A'
        )
        ORDER BY
            DSOCORR
        OFFSET (:PAGE - 1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("DSOCORR", $params['search']);
        $stmt->bindValue("IDCONTROLAOBS", $params['idControlaObs']);
        $stmt->bindValue("PAGE", $params['page'] ?? 1);
        $stmt->bindValue("LIMIT", $params['limit'] ?? 100);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        return [$result, $total];
    }

    public function checkSkuExists($params) {
        $sql = "
            SELECT CDPRODUTO
                FROM PRODUTO
            WHERE
                CDPRODUTO = :CDPRODUTO
            AND
                LENGTH(CDPRODUTO) = 10
        ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDPRODUTO", $params);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();
        if (empty($result)) {
            throw new \Exception(
                "Não existe nenhum item correspondente ao código inserido.\n".
                "Verifique o código digitado e tente novamente"
            );
        } else {
            return $result[0]['CDPRODUTO'];
        }
    }

    public function checkOccurrenceExists($params) {
        list($CDGRPOCOR, $CDOCORR) = explode('|', $params['occurrence']);

        $sql = "
        SELECT CDPRODUTO
            FROM OCORRENCIA
        WHERE
            CDGRPOCOR = :CDGRPOCOR
        AND
            CDOCORR = :CDOCORR
        AND (
            IDCONTROLAOBS = :IDCONTROLAOBS
        OR
            IDCONTROLAOBS IN ('R', 'O')
        )
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDGRPOCOR", $CDGRPOCOR);
        $stmt->bindValue("CDOCORR", $CDOCORR);
        $stmt->bindValue("IDCONTROLAOBS", $params['idControlaObs']);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        if (count($result) === 0) {
            throw new \Exception(
                "Não existe uma ocorrencia associada ao codigo informado. ".
                "Verifique o código digitado e tente novamente"
            );
        } elseif($result[0]['CDPRODUTO'] === null) {
            throw new \Exception("Ocorrência não possui um código de produto");
        } else {
            return $result[0]['CDPRODUTO'];
        }
    }

    public function getSubsidiaryByIntegrationCode($CDINTEGRACAO) {
        $sql = "
            SELECT DISTINCT
                I.CDFILIAL,
                I.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL
            FROM
                INTEGRACAOFILI I
            INNER JOIN
                FILIAL F ON I.CDFILIAL = F.CDFILIAL
            WHERE
                I.CDINTEGRACAO = :CDINTEGRACAO
            ORDER BY
                I.CDFILIAL
            ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDINTEGRACAO", $CDINTEGRACAO);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }

    public function getMerchantId($idIntegracao) {
        $sql = "
            SELECT
                f.CDINTEGRACAO,
                f.CDFILIAL,
                f.IDTOKEN
            FROM
                INTEGRACAOFILI f
            JOIN
                INTEGRACAO i ON f.CDINTEGRACAO = i.CDINTEGRACAO
            WHERE
                (INSTR(:IDINTEGRACAO, i.IDINTEGRACAO) > 0 OR 'T' = :IDINTEGRACAO)
            ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDINTEGRACAO", $idIntegracao);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }

    public function getLastModifiedDate($cdIntegracao, $menuId) {
        $sql = "
            SELECT
                TO_CHAR(DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS') AS DTINCLUSAO
            FROM
                LOGALTPRODCARD
            WHERE
                CDINTEGRACAO = :CDINTEGRACAO
            AND
                IDEXTCARD = :IDEXTCARD
            ORDER BY
                TO_DATE(DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS') DESC
            FETCH FIRST 1 ROWS ONLY
            ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDINTEGRACAO", $cdIntegracao);
        $stmt->bindValue("IDEXTCARD", $menuId);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();
        return $result[0]['DTINCLUSAO'] ?? "";
    }

    public function logItemAlteration($params) {
        try {
            $this->connection->beginTransaction();

            foreach ($params['items'] as $item) {
                $sql = "
                    INSERT INTO LOGALTPRODCARD
                        (CDINTEGRACAO, CDFILIAL, IDEXTCARD, CDOPERADOR, CDPRODUTO,
                        IDEXTPROD, NMPRODCARD, DSALTERA, CDEXTANT, CDEXTATU, VRPRECANT, VRPRECATU, TPALTERA)
                    VALUES
                        (:CDINTEGRACAO, :CDFILIAL, :IDEXTCARD, :CDOPERADOR, :CDPRODUTO,
                        :IDEXTPROD, :NMPRODCARD, :DSALTERA, :CDEXTANT, :CDEXTATU, :VRPRECANT, :VRPRECATU, :TPALTERA)
                ";
                $stmt = $this->connection->prepare($sql);
                $description = $this->describeAlteration($item);
                $stmt->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
                $stmt->bindValue("CDOPERADOR", $params['cdOperador']);
                $stmt->bindValue("CDFILIAL", $params['cdFilial']);
                $stmt->bindValue("IDEXTCARD", $item['menuId']);
                $stmt->bindValue("CDPRODUTO", $item['cdProduto']);
                $stmt->bindValue("IDEXTPROD", $item['id']);
                $stmt->bindValue("NMPRODCARD", $item['name']);
                $stmt->bindValue("DSALTERA", $description);
                $stmt->bindValue("CDEXTANT", $item['initialSku']);
                $stmt->bindValue("CDEXTATU", $item['finalSku']);
                $stmt->bindValue("VRPRECANT", $item['initialPrice']);
                $stmt->bindValue("VRPRECATU", $item['finalPrice']);
                $stmt->bindValue("TPALTERA", $item['type']);
                $stmt->executeQuery();
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw new \Exception("Não foi possível salvar as alterações feitas nos itens: " . $e->getMessage());
        }
    }

    private function formatPrice($price) {

        $formattedPrice = 'R$'.number_format($price, 2, ',', '.');

        return $formattedPrice;
    }

    private function describeAlteration($params) {
        $description = '';

        if ($params['initialPrice'] != $params['finalPrice']) {
            $description = 'Preço alterado de '.$this->formatPrice($params['initialPrice']).
                            ' para '.$this->formatPrice($params['finalPrice']);
        }

        if ($params['initialSku'] != $params['finalSku']) {
            $description .= 'Código externo alterado de '.$params['initialSku'].
            ' para '.$params['finalSku'].'';
        }
        if($params['type'] === 'I'){
            $description .= ' no item "'.$params['name'].'" ';

        } else {
            $description .= ' no complemento "'.$params['name'].'" ';
        }
        return $description;
    }

    public function getAlterationLogs($params) {
        $totalRows = "
        SELECT COUNT(*) as TOTAL
        FROM
            LOGALTPRODCARD
            WHERE
            CDINTEGRACAO = :CDINTEGRACAO
        AND
            IDEXTCARD = :IDEXTCARD
        ";

        $totalRows = $this->connection->prepare($totalRows);
        $totalRows->bindValue("IDEXTCARD", $params['menuId']);
        $totalRows->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
        $totalRows = $totalRows->executeQuery();
        $totalRows = $totalRows->fetchAllAssociative();
        $totalRows = intval($totalRows[0]["TOTAL"]);
        $sql = "
        SELECT
            L.CDLOG,
            TO_CHAR(L.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS') AS DTINCLUSAO,
            L.DSALTERA,
            L.NMPRODCARD,
            O.NMOPERADOR || ' ('||L.CDOPERADOR||')' AS AUTOR,
            CASE WHEN L.VRPRECANT <> L.VRPRECATU THEN L.VRPRECANT ELSE NULL END AS ANTERIOR_VRPREC,
            CASE WHEN L.CDEXTANT <> L.CDEXTATU THEN L.CDEXTANT ELSE NULL END AS ANTERIOR_CDEXT,
            CASE WHEN L.VRPRECANT <> L.VRPRECATU THEN L.VRPRECATU ELSE NULL END AS ATUAL_VRPREC,
            CASE WHEN L.CDEXTANT <> L.CDEXTATU THEN L.CDEXTATU ELSE NULL END AS ATUAL_CDEXT
        FROM
            LOGALTPRODCARD L
        INNER JOIN
            OPERADOR O ON L.CDOPERADOR = O.CDOPERADOR
        WHERE
            L.CDINTEGRACAO = :CDINTEGRACAO
        AND
            L.IDEXTCARD = :IDEXTCARD
        AND (
            :SEARCH IS NULL
            OR (
                LOWER(L.DSALTERA) LIKE LOWER('%'||:SEARCH||'%')
                OR LOWER(O.NMOPERADOR) LIKE LOWER('%'||:SEARCH||'%')
                OR LOWER(TO_CHAR(L.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS')) LIKE LOWER('%'||:SEARCH||'%')
            )
        )
        ORDER BY
            L.DTINCLUSAO DESC
        OFFSET (:PAGE-1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDEXTCARD", $params['menuId']);
        $stmt->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
        $stmt->bindValue("PAGE", $params['page']?? 1);
        $stmt->bindValue("SEARCH", $params['search'] ?? null);
        $stmt->bindValue("LIMIT", $params['limit']??15);
        $result = $stmt->executeQuery();
        $data = $result->fetchAllAssociative();
        return [$data, $totalRows];
    }

    public function getAuthToken($params) {
        $sql = "SELECT AUTHTOKEN FROM INTEGRACAO WHERE IDINTEGRACAO = :IDINTEGRACAO";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDINTEGRACAO", $params['IDINTEGRACAO']);
        $result = $stmt->executeQuery()->fetchAssociative();
        if (empty($result)) {
            return null;
        }
        return $result['AUTHTOKEN'];
    }

    public function getIdSenhaTokenGeral($idIntegracao) {
        $sql = "
                SELECT IDSENHATOKENGERAL
                    FROM INTEGRACAO
                WHERE IDINTEGRACAO = :IDINTEGRACAO
                ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDINTEGRACAO", $idIntegracao);
        $result = $stmt->executeQuery()->fetchAssociative();
        if (empty($result)) {
            return null;
        }
        return $result["IDSENHATOKENGERAL"];
    }
}
