<?php
namespace App\Service;

use App\Report\ReportBuilder;

class Report {
  private $reportFactory;
  private $util;
  private $orderService;

  public function __construct(\App\Factory\Report $reportFactory,
                              \App\Util\Util $util,
                              \App\Service\Order $orderService) {
      $this->reportFactory = $reportFactory;
      $this->util = $util;
      $this->orderService = $orderService;
  }

  public function generateReport($params) {
    $data = [];
    switch($params['report']) {
      case 'order':
        $order = $this->prepareOrderReport($params['paramsFilter']);
        $data['dtInicial'] = isset($params['paramsFilter']['dtInicial']) ? $params['paramsFilter']['dtInicial'] : '';
        $data['dtInicial'] = isset($params['paramsFilter']['dtFinal']) ? $params['paramsFilter']['dtFinal'] : '';
        $data = array_merge($order, $this->formatedData($params));
        return ReportBuilder::export($data);
      case 'production-order':
        $data = $this->prepareOrderProduction($params);
        $data = array_merge($data, $this->formatedData($params));
        return ReportBuilder::export($data);
        default:
          throw new \Exception('INVALID_NAME_REPORT');
    }
  }

  private function prepareOrderReport($paramsFilter) {
    $order = $this->orderService->history($paramsFilter);
    return ['order' => $this->formatOrder($order)];
  }

  private function formatOrder($order) {
    $formattedOrders = array();
    foreach ($order as $items) {
        $formattedOrderItem = array(
          'IDSTCOMANDA'     => $this->util->formatStatus($items['IDSTCOMANDA'], $items['IDSINCMSDE']),
          'IDORGCMDVENDA'   => $this->util->commandSourceDictionary($items['IDORGCMDVENDA']),
          'IDSINCMSDE'      => $this->util->formatSynced($items['IDSINCMSDE']),
          'VRPEDIDO'        => $this->util->formatMoney($items['VRPEDIDO']),
          'VRTOTALPRODUTOS' => $this->util->formatMoney($items['VRTOTALPRODUTOS']),
          'VRDESCOMANDA'    => $this->util->formatMoney($items['VRDESCOMANDA']),
          'ACRESCIMO'       => $this->util->formatMoney($items['ACRESCIMO']),
          'VRACRCOMANDA'    => $this->util->formatMoney($items['VRACRCOMANDA']),
        );
      $formattedOrders[] = array_merge($items, $formattedOrderItem);
    }
    return $formattedOrders;
  }

  private function formatedData($params) {
    $data['title']  = isset($params['title']) ? $params['title'] : '';
    $data['report'] = isset($params['report']) ? $params['report'] : '';
    $data['format'] = isset($params['format']) ? $params['format'] : '';
    $data['visibleColumns'] = isset($params['visibleColumns']) ? $params['visibleColumns'] : '';
    $data['period'] = '';
    if (isset($params['dtInicial']) && isset($params['dtFinal'])) {
      $data['period'] = $params['dtInicial'] . ' - ' . $params['dtFinal'];
    }
    return $data;
  }

  private function prepareOrderProduction($params = []) {
    $orderProduction = $this->reportFactory->orderProduction($params);
    if (empty($orderProduction)) throw new \Exception('NO_REQUEST_GENERATE_REPORT_FOUND');

    return $this->groupOrdersByProduct($orderProduction);
  }

  private function groupOrdersByProduct($data) {
    $result = [];
    $items = [];
    foreach ($data as $item) {
        $itemData = [
            'PRODUTO'       => $item['PRODUTO'],
            'PEDIDO'        => $item['PEDIDO'],
            'IDORGCMDVENDA' => $this->util->commandSourceDictionary($item['IDORGCMDVENDA']),
            'ENDERECO'      => $item['ENDERECO'],
            'OBSERVACAO'    => $this->processObservations($item['OBSERVACAO'], 60),
            'OBSERVACAOCMD' => $this->processObservations($item['OBSERVACAOCMD'], 60),
            'DATA'          => $item['DATA'],
            'HORARIO'       => $item['HORARIO'],
            'QUANTIDADE'    => $item['QUANTIDADE'],
            'CLIENTE'       => $item['CLIENTE'],
        ];
        $product = $item['PRODUTO'];
        $date = $item['DATA'];
        if(isset($items[$date])) {
          $items[$date][] = $itemData;
        } else {
          $items[$date] = [$itemData];
        }
    }
    $newItems = [];
    foreach($items as &$dateGroup) {
      usort($dateGroup, fn($a, $b) => $a['HORARIO'] <=> $b['HORARIO']);
      $productGroup = [];
      foreach($dateGroup as $item) {
        $product = $item['PRODUTO'];
        if(isset($productGroup[$product])) {
          $productGroup[$product][] = $item;
        } else {
          $productGroup[$product] = [$item];
        }
      }
      $newItems = array_merge($newItems, array_values($productGroup));
    }

    $products = array_merge(...array_values($newItems));

    $result['PRODUTOS'] = array_chunk($products, 3);
    $result['SETOR'] = $data[0]['SETOR'];
    foreach($result['PRODUTOS'] as &$groupProd) {
      foreach($groupProd as $idx => $prod) {
        $curDate = $prod['DATA'];
        $curProd = $prod['PRODUTO'];
        $result['PROD_QTDE'][$curDate] = $result['PROD_QTDE'][$curDate] ?? [];
        $result['PROD_QTDE'][$curDate][$curProd] = $result['PROD_QTDE'][$curDate][$curProd] ?? 0;
        $result['PROD_QTDE'][$curDate][$curProd] += $prod['QUANTIDADE'];

        if ($idx === 0) {
          $groupProd[0]['ROW_SPAN'] = 2;
          continue;
        }
        $isEqualDateStart = $curDate === $groupProd[0]['DATA'];
        $isEqualDateBefore = $curDate === $groupProd[$idx-1]['DATA'];
        $isEqualProdStart = $curProd === $groupProd[0]['PRODUTO'];
        $isEqualProdBefore = $curProd === $groupProd[$idx-1]['PRODUTO'];
        if ($isEqualDateStart){
          $groupProd[0]['ROW_SPAN']++;
          if (!$isEqualProdStart && !$isEqualProdBefore) { // new group header
            $groupProd[0]['ROW_SPAN']++;
          }
        } else if ($isEqualDateBefore) {
          $groupProd[$idx-1]['ROW_SPAN'] = $groupProd[$idx-1]['ROW_SPAN'] ?? 2;
          $groupProd[$idx-1]['ROW_SPAN']++;
          if (!$isEqualProdBefore) { // new group header
            $groupProd[$idx-1]['ROW_SPAN']++;
          }
        } else { // new date
          $groupProd[$idx]['ROW_SPAN'] = 2;
        }
      }
    }
    return $result;
  }

  private function processObservations($observation, $maxLength) {
    if (!isset($observation) || is_null($observation)) return '';

    return implode("\n", array_map(function($obs) use ($maxLength) {
        $obs = trim($obs);
        return strlen($obs) > $maxLength ? substr($obs, 0, $maxLength) . '...' : $obs;
    }, explode(';', $observation)));
  }

  public function getUnit($params) {
    return $this->reportFactory->getUnit($params);
  }

  public function getSector($params) {
    return $this->reportFactory->getSector($params);
  }
}
